package WWW::Suffit;
use warnings;
use strict;
use utf8;

=encoding utf8

=head1 NAME

WWW::Suffit - The Suffit core library

=head1 SYNOPSIS

    use WWW::Suffit;

=head1 DESCRIPTION

This library provides common functionality for all sub projects of the Suffit metasystem

=head1 DEPENDENCIES

L<Mojolicious>

=head1 HISTORY

See C<Changes> file

=head1 TO DO

See C<TODO> file

=head1 SEE ALSO

L<Mojolicious>

=head1 AUTHOR

Serż Minus (Sergey Lepenkov) L<https://www.serzik.com> E<lt>abalama@cpan.orgE<gt>

=head1 COPYRIGHT

Copyright (C) 1998-2023 D&D Corporation. All Rights Reserved

=head1 LICENSE

This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

See C<LICENSE> file and L<https://dev.perl.org/licenses/>

=cut

our $VERSION = '1.07';

1;

__END__
